"""booking

Revision ID: 7e9944b430d6
Revises: 1d95eb10ea20
Create Date: 2023-08-19 14:25:40.206187

"""
import sqlalchemy as sa

from alembic import op
from sqlalchemy.dialects import postgresql


# revision identifiers, used by Alembic.
revision = "7e9944b430d6"
down_revision = "1d95eb10ea20"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("customers", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column(
        "customers", "phone", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "customers", "address", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column("devices", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column(
        "food_tables", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "menu_categories", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "modifier_categories", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column("modifiers", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    overviewstatus = sa.Enum("running", "printed", name="overviewstatus")
    overviewstatus.create(op.get_bind())
    op.execute("ALTER TABLE overview ALTER COLUMN status TYPE overviewstatus USING status::text::overviewstatus;")
    op.alter_column(
        "overview",
        "status",
        existing_type=sa.VARCHAR(length=255),
        type_=overviewstatus,
        existing_nullable=False,
    )
    op.alter_column(
        "permissions", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column("printers", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column(
        "printers", "address", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "printers", "cut_code", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "product_versions", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column(
        "product_versions", "units", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column("regimes", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column("regimes", "header", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column("regimes", "prefix", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column("roles", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column(
        "sale_categories", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column("sections", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column("settings", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column("settings", "data", existing_type=postgresql.JSON(astext_type=sa.Text()), nullable=False)
    op.alter_column(
        "settle_options", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False
    )
    op.alter_column("taxes", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column("users", "name", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=False)
    op.alter_column("users", "password", existing_type=sa.VARCHAR(length=60), type_=sa.Text(), existing_nullable=False)
    op.alter_column(
        "vouchers", "narration", existing_type=sa.VARCHAR(length=1000), type_=sa.Text(), existing_nullable=True
    )
    op.alter_column("vouchers", "reason", existing_type=sa.VARCHAR(length=255), type_=sa.Text(), existing_nullable=True)

    guestbooktype = sa.Enum("walk_in", "booking", "arrived", name="guestbooktype")
    guestbooktype.create(op.get_bind())
    gbt = sa.table(
        "guest_book",
        sa.column("last_edit_date", sa.DateTime()),
        sa.column("arrival_date", sa.DateTime()),
        sa.column("type", guestbooktype),
    )

    with op.batch_alter_table("guest_book") as batch_op:
        batch_op.add_column(sa.Column("booking_date", sa.DateTime(), nullable=True))
        batch_op.alter_column("creation_date", new_column_name="last_edit_date")
        batch_op.add_column(sa.Column("arrival_date", sa.DateTime(), nullable=True))
        batch_op.add_column(sa.Column("notes", sa.Text(), nullable=True))
        batch_op.add_column(sa.Column("type", guestbooktype, nullable=True))
    op.execute(gbt.update().values(arrival_date=sa.text("last_edit_date")))
    op.execute(gbt.update().values(type="walk_in"))
    with op.batch_alter_table("guest_book") as batch_op:
        batch_op.alter_column("type", existing_type=guestbooktype, nullable=False)
        batch_op.create_check_constraint("ck_guest_book_nulls", sa.text("num_nonnulls(booking_date, arrival_date) > 0"))

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column("vouchers", "reason", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=True)
    op.alter_column(
        "vouchers", "narration", existing_type=sa.Text(), type_=sa.VARCHAR(length=1000), existing_nullable=True
    )
    op.alter_column("users", "password", existing_type=sa.Text(), type_=sa.VARCHAR(length=60), existing_nullable=False)
    op.alter_column("users", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column("taxes", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column(
        "settle_options", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column("settings", "data", existing_type=postgresql.JSON(astext_type=sa.Text()), nullable=True)
    op.alter_column("settings", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column("sections", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column(
        "sale_categories", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column("roles", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column("regimes", "prefix", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column("regimes", "header", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column("regimes", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column(
        "product_versions", "units", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column(
        "product_versions", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column(
        "printers", "cut_code", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column(
        "printers", "address", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column("printers", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column(
        "permissions", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column(
        "overview",
        "status",
        existing_type=sa.Enum("running", "printed", name="overviewstatus"),
        type_=sa.VARCHAR(length=255),
        existing_nullable=False,
    )
    op.alter_column("modifiers", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column(
        "modifier_categories", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column(
        "menu_categories", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.add_column("guest_book", sa.Column("creation_date", postgresql.TIMESTAMP(), autoincrement=False, nullable=False))
    op.drop_column("guest_book", "type_")
    op.drop_column("guest_book", "notes")
    op.drop_column("guest_book", "last_edit_date")
    op.drop_column("guest_book", "date")
    op.alter_column(
        "food_tables", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column("devices", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    op.alter_column(
        "customers", "address", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=True
    )
    op.alter_column(
        "customers", "phone", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False
    )
    op.alter_column("customers", "name", existing_type=sa.Text(), type_=sa.VARCHAR(length=255), existing_nullable=False)
    # ### end Alembic commands ###
