import uuid

from datetime import datetime
from typing import TYPE_CHECKING

from sqlalchemy import DateTime, ForeignKey, UniqueConstraint, Uuid, text
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .device import Device
    from .user import User


@reg.mapped_as_dataclass(unsafe_hash=True)
class LoginHistory:
    __tablename__ = "login_history"
    __table_args__ = (UniqueConstraint("user_id", "device_id", "date"),)
    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, server_default=text("gen_random_uuid()"))
    user_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("users.id"), nullable=False)
    device_id: Mapped[uuid.UUID] = mapped_column(
        Uuid,
        ForeignKey("devices.id"),
        nullable=False,
    )
    date: Mapped[datetime] = mapped_column(DateTime(), nullable=False)

    device: Mapped["Device"] = relationship(back_populates="login_history")
    user: Mapped["User"] = relationship(back_populates="login_history")

    def __init__(self, user_id=None, device_id=None, date=None, id_=None):
        self.user_id = user_id
        self.device_id = device_id
        self.date = datetime.utcnow() if date is None else date
        self.id = id_
