import uuid

from typing import TYPE_CHECKING, List

from sqlalchemy import Boolean, Integer, Text, Uuid, text
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .product_version import ProductVersion


@reg.mapped_as_dataclass(unsafe_hash=True)
class MenuCategory:
    __tablename__ = "menu_categories"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, server_default=text("gen_random_uuid()"))
    name: Mapped[str] = mapped_column(Text, nullable=False, unique=True)

    is_active: Mapped[bool] = mapped_column(Boolean, nullable=False)
    is_fixture: Mapped[bool] = mapped_column(Boolean, nullable=False)
    sort_order: Mapped[int] = mapped_column(Integer, nullable=False)

    products: Mapped[List["ProductVersion"]] = relationship(back_populates="menu_category")

    def __init__(self, name, is_active, sort_order, is_fixture=False, id_=None):
        self.name = name
        self.is_active = is_active
        self.sort_order = sort_order
        self.is_fixture = is_fixture
        self.id = id_
