import uuid

from datetime import date, datetime

from pydantic import BaseModel, ConfigDict, field_serializer, field_validator

from . import Daf, to_camel


class MeItem(BaseModel):
    id_: uuid.UUID
    name: str
    price: Daf
    average: Daf
    sale_category: str
    menu_category: str
    quantity: Daf
    sales: Daf
    quantity_percent: Daf
    sales_percent: Daf
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class MeReport(BaseModel):
    start_date: date
    finish_date: date
    amounts: list[MeItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("start_date", mode="before")
    @classmethod
    def parse_start_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("start_date")
    def serialize_start_date(self, value: date, _info):
        return value.strftime("%d-%b-%Y")

    @field_validator("finish_date", mode="before")
    @classmethod
    def parse_finish_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("finish_date")
    def serialize_finish_date(self, value: date, _info):
        return value.strftime("%d-%b-%Y")
